	PROCESSOR 16F876A
	INCLUDE <P16F876A.INC>

    errorlevel -302  ;suppress bank  warnings 
	__config _HS_OSC & _WDT_OFF & _PWRTE_OFF & _CP_OFF & _LVP_OFF & _BODEN_OFF


w_temp   EQU 0x70  ; save W register contents in common memory 0x70 to 0x7f all banks
status_temp EQU 0x71 ; save STATUS register


	ORG 0x0000
	GOTO MAIN

	ORG 0x0004 ;serial INTERRUPT VECTOR
	GOTO   INTERRUPT_HANDLER

MAIN:

    call initSerial
;    call setSerialInterrupt  ;For interrupt version,disable for polled version
    

MAIN_LOOP: ;CONTINOUS LOOP

    banksel	PIR1  ;bank 0

	BTFSS PIR1,RCIF ; check fo byte in
    goto noDataWaiting
    movf RCREG,w      ;For polled version,disable for interrupt version
    call sendChar     ;For polled version,disable for interrupt version

noDataWaiting:

	GOTO MAIN_LOOP ;IF NOT GOTO CONTINOUS LOOP
;-------------- end main code




;-------------------------------------------------------


sendChar:    ;send the char in W
    banksel	TXSTA ;bank 1
sendCharWait:
	BTFSS TXSTA,TRMT ; wait til clear
    goto sendCharWait
    banksel	TXREG ;bank 0
    movwf TXREG
    return;

;--------------------------------------------------------

initSerial:
    banksel	SPBRG  ;; and TRISC + txsta
	movlw 0xc0
    IORWF TRISC,f	;set bits 6+7 to enable serial bits

	;---CONFIGURE SPBRG FOR DESIRED BAUD RATE
    
	;MOVLW .51 ; 51 decimal 129  9600 @4mHz  
	MOVLW .129 ;decimal 129  9600 @20mHz 
	MOVWF SPBRG 

	;---CONFIGURE TXSTA
	MOVLW B'00100110' ;CONFIGURE TXSTA AS :8bit ,high speed enable,
	MOVWF TXSTA       ;  Transmit Shift Register Status bit =1 when tx reg empty

    banksel	RCSTA

	MOVLW 0x90 ;ENABLE SERIAL PORT
	MOVWF RCSTA ;RECEIVE STATUS REG
    return

;-------------------------------------------------------------
setSerialInterrupt:
	;---CONFIGURE PERIPHERAL INTERRUPTS

    banksel	PIE1  ;bank1
    bsf PIE1, RCIE 
    bsf INTCON, GIE ; enable global interrupt
    bsf INTCON, PEIE ; enable global interrupt
    return
;--------------------------------------------------------------
INTERRUPT_HANDLER:
    movwf   w_temp     ; save W register contents
	movf    STATUS, w   ; move status register into W 
	movwf	status_temp  ; save STATUS register

    banksel RCREG  ;bank0
	BTFSS PIR1,RCIF ; only handle serial irq
	GOTO EXIT_INT   ;exit

    banksel RCREG  ;bank0
    movf RCREG,w   ;echo char back
    call sendChar
EXIT_INT:
    movf    status_temp,w     ; retrieve STATUS register /bank settings
	movwf	STATUS            ; restore STATUS register 
	swapf   w_temp,f
	swapf   w_temp,w          ; restore W register 
   	RETFIE


	END

